unit Testform;

(*                                   *)
(* 3DR DEMONSTRATION FORM            *)
(* By Steven E. Hugg, hugg@pobox.com *)
(* Use at own risk, etc.             *)
(*                                   *)

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, _3DR, ExtCtrls;

type
  TForm1 = class(TForm)
    Timer1: TTimer;
    procedure FormCreate(Sender: TObject);
    procedure FormPaint(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure Timer1Timer(Sender: TObject);
  private
    { Private declarations }
    R3dRC : TR3dHandle;
    hdc3dr : HDC;
    frame : integer;
    procedure Render;
    procedure RenderTri(cx, cy, rot, scale : single);
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

const
  MEDIUM_GREY = $FF808080;
  ALPHA = $8000;
  numtri = 50;
  triscale = 10;
  numsides = 3;

var
  tri : array[0..numtri-1] of record
    xx,yy,rr : single;
    xvel, rvel : single;
  end;

function DwordToFixedPt(D : longint) : TFixed32;
begin
  Result := TFixed32((D and $FFFF) shl 16);
end;

function FloatToFixedPt(F : single) : TFixed32;
begin
  Result := TFixed32(round(F*65536.0));
end;

const
  VertexData : array[0..2] of TR3dFixedVtxClr = (
  (* x y z  r      g      b       a  *)
    (x:0; y:0; z:0; r:$FFFF; g:$0000; b:$0000; a:ALPHA),
    (x:0; y:0; z:0; r:$0000; g:$FFFF; b:$0000; a:ALPHA),
    (x:0; y:0; z:0; r:$0000; g:$0000; b:$FFFF; a:ALPHA)
  );

procedure TForm1.FormCreate(Sender: TObject);
var
  i : integer;
begin
  hDC3DR := GetDC(Handle);
  R3dRC := R3dCreateRC(TR3dHandle(hDC3DR), 0);
  if R3dRC = 0 then
    raise Exception.Create('Could not create 3DR context');
  R3dSetState(R3DRC, R3D_STA_BG_COLOR, {MEDIUM_GREY}0);
  R3dSetState(R3DRC, R3D_STA_ENABLES,  R3D_SE_SHADING);
  R3dSetState(R3DRC, R3D_STA_ENABLES,  R3D_SE_CLIPPING);
{  R3dSetState(R3DRC, R3D_STA_ENABLES,  R3D_SE_ANTIALIASING);}
{  R3dSetState(R3DRC, R3D_STA_ENABLES,  R3D_SE_Z_BUFFERING);
  R3dSetState(R3DRC, R3D_STA_ENABLES,  R3D_SE_BLENDING);
  R3dSetState(R3DRC, R3D_STA_SRC_BLEND_FUNCTION,  R3D_BF_SRC_ALPHA);
  R3dSetState(R3DRC, R3D_STA_DST_BLEND_FUNCTION,  R3D_BF_SRC_ALPHA_INV);}
  FillChar(tri, sizeof(tri), 0);
  for i := 0 to numtri-1 do with tri[i] do begin
    xx := 0;
    yy := random(Height);
    rr := random*pi;
    xvel := random;
    rvel := random*pi/16;
  end;
  Timer1.Enabled := true;
end;

procedure TForm1.RenderTri(cx, cy, rot, scale : single);
var
  r,s1,c1 : single;
  i : integer;
begin
  for i := 0 to NumSides-1 do begin
    r := rot + (3.141592*2/NumSides)*i;
    s1 := sin(r)*scale;
    c1 := cos(r)*scale;
    with VertexData[i] do begin
      x := FloatToFixedPt(cx+s1);
      y := FloatToFixedPt(cy+c1);
      z := i;
    end;
  end;
  R3dPrimitive(   R3DRC,
                  R3D_PKT_POLYGON,
                  R3D_VI_COLOR,
                  3,
                  sizeof(TR3dFixedVtxClr),
                  @VertexData);
end;

procedure TForm1.Render;
var
  i  :integer;
begin
  R3dClearBuffers(R3DRC, R3D_CI_DRAW_BUFFER);

  for i := 0 to numtri-1 do with tri[i] do begin
    RenderTri(xx, yy, rr, triscale);
    xx := xx + xvel;
    rr := rr + rvel;
  end;

  R3dSwapBuffers( R3DRC, 0);
end;

procedure TForm1.FormPaint(Sender: TObject);
begin
  Render;
end;

procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  R3dDeleteRC(R3DRC);
  ReleaseDC(Handle, hDC3DR);
end;

procedure TForm1.Timer1Timer(Sender: TObject);
begin
  inc(frame);
  Render;
end;

end.
